* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=" " PageSize=MAX LineSize=MAX SPOOL;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; *ODS LISTING;

***********************************************************************************;
*******     MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR THIS EXAMPLE        *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use TotalR2 macro;
* DV =        Case-sensitive name of dependent variable (required);
* PredFewer = Name of OUTPM= data file of predicted outcomes for first model (required);
* PredMore =  Name of OUTPM= data file of predicted outcomes for second model (optional);
%MACRO TotalR2(DV=, Pred1=, Pred2=);
PROC CORR NOPRINT NOSIMPLE DATA=&Pred1. OUTP=Corr1; VAR pred &DV.; RUN;
DATA Corr1; LENGTH Name $30.; SET Corr1; Name="&Pred1.";
     IF Pred=. THEN Pred=0; PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; 
     KEEP Name PredCorr TotalR2; RUN;
%IF %LENGTH(&Pred2.)>0 %THEN %DO;
PROC CORR NOPRINT NOSIMPLE DATA=&Pred2. OUTP=Corr2; VAR pred &DV.; RUN;
DATA Corr2; LENGTH Name $30.; SET Corr2; Name="&Pred2."; 
     IF Pred=. THEN Pred=0; PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; 
     KEEP Name PredCorr TotalR2; RUN;
DATA CorrCompare; LENGTH Name $30.; SET Corr1 Corr2; 
     TotalR2Diff=TotalR2-LAG1(TotalR2);
     KEEP Name PredCorr TotalR2 TotalR2Diff; RUN; 
%END;
%IF %LENGTH(&Pred2.)>0 %THEN %LET ToPrint=CorrCompare; 
     %ELSE %LET ToPrint=Corr1;
TITLE8 "Total-R2 Per Model";
%IF %LENGTH(&Pred2.)>0 %THEN %DO;
TITLE9 "Change in Total-R2 for &Pred1. vs. &Pred2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;   
%MEND TotalR2;

* To use PseudoR2 macro;
* Ncov =     TOTAL # entries in covariance parameter estimates table;
* CovFewer = Name of ODS CovParms table for nested baseline model (required);
* CovMore1 = Name of ODS CovParms table for comparison model1 (required);
* CovMore2 = Name of ODS CovParms table for comparison model2 (optional);
%MACRO PseudoR2(NCov=, CovFewer=, CovMore1=, CovMore2=);
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&CovFewer."; RUN;
DATA &CovMore1.; LENGTH Name $30.; SET &CovMore1.; Name="&CovMore1."; RUN;
DATA CovCompare1; LENGTH Name $30.; SET &CovFewer. &CovMore1.; RUN;
DATA CovCompare1; SET CovCompare1; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
%IF %LENGTH(&CovMore2.)>0 %THEN %DO; %LET lagchange=%EVAL(&Ncov.*2);
DATA &CovMore2.; LENGTH Name $30.; SET &CovMore2.; Name="&CovMore2."; RUN;
DATA CovCompare2; LENGTH Name $30.; SET &CovFewer. &CovMore2.; RUN;
DATA CovCompare2; SET CovCompare2; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare1 CovCompare2;
     PseudoR2Change=PseudoR2-(LAG&lagchange(PseudoR2)); RUN;
%END; %IF %LENGTH(&CovMore2.)>0 %THEN %LET ToPrint=CovCompare; 
     %ELSE %LET ToPrint=CovCompare1;
DATA &ToPrint.; SET &ToPrint.; 
     IF CovParm IN("UN(2,1)","UN(3,1)","UN(3,2)","UN(4,1)","UN(4,2)","UN(4,3)",
                   "UN(5,1)","UN(5,2)","UN(5,3)","UN(5,4)",
                   "UN(6,1)","UN(6,2)","UN(6,3)","UN(6,4)","UN(6,5)") THEN DELETE; 
     DROP Zvalue ProbZ; RUN;
TITLE8 "Pseudo-R2 Relative to &CovFewer.";
%IF %LENGTH(&CovMore2.)>0 %THEN %DO;
TITLE9 "Change in Pseudo-R2 for &CovMore1. vs. &CovMore2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;
%MEND PseudoR2;

* To use FitTest macro;
* FitFewer = name of infocrit table for nested model;
* FitMore  = name of infocrit table for comparison model;
%MACRO FitTest(FitFewer=, FitMore=);
DATA &FitFewer.; LENGTH Name $30.; SET &FitFewer.; Name="&FitFewer."; RUN;
DATA &FitMore.;  LENGTH Name $30.; SET &FitMore.;  Name="&FitMore.";  RUN;
DATA FitCompare; LENGTH Name $30.; SET &FitFewer. &FitMore.; RUN;
DATA FitCompare; SET FitCompare; DevDiff=Lag1(Neg2LogLike)-Neg2LogLike;
     DFdiff=Parms-LAG1(Parms); Pvalue=1-PROBCHI(DevDiff,DFdiff);
     DROP AICC HQIC CAIC; RUN;
TITLE2 "Likelihood Ratio Test for &FitFewer. vs. &FitMore.";
PROC PRINT NOOBS DATA=FitCompare; RUN; TITLE2;
%MEND FitTest;


***********************************************************************************;
*******           BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 5           *******;
*******               CHANGE "filesave" TO YOUR FOLDER PATH                 *******;
***********************************************************************************;

* Define global variable for file location to be replaced in code below;
* \\Client\ precedes actual path when using UIowa Virtual Desktop;
%LET filesave = C:\Dropbox\23_PSQF6272\PSQF6272_Example5;

* Import trimmed example data excel data file from sheet "Hox" into SAS;
PROC IMPORT DATAFILE="&filesave.\Example5_Data.xlsx" OUT=work.Example5 DBMS=XLSX REPLACE; 
     SHEET="Hox"; GETNAMES=YES; RUN;

* Add labels to original variables;
DATA work.Example5; SET work.Example5;
LABEL childID=    "childID: Child ID"
      PschoolID=  "PschoolID: Primary School ID"
      SschoolID=  "SschoolID: Secondary School ID"
      achieve=    "achieve: Child Achievement Outcome"
      ses=        "ses: Child Socio-Economic Status"
      Pdenom=     "Pdenom: Primary School Denomination"
      Sdenom=     "Sdenom: Secondary School Denomination"; RUN;

* Open external file to save results to;
ODS RTF FILE="&filesave.\Example5_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;

TITLE "SAS Descriptives for Child Variables";
PROC MEANS NDEC=2 DATA=work.Example5; VAR achieve ses; RUN;

* Get means per primary school for child variables;
PROC SORT DATA=work.Example5; BY PschoolID; RUN;
PROC MEANS NOPRINT DATA=work.Example5; BY PschoolID Pdenom; VAR achieve ses; 
     OUTPUT OUT=work.Primary MEAN(achieve ses)=PMachieve PMses; RUN;
DATA work.Primary; SET work.Primary;
     PrimN = _FREQ_;
     LABEL PMachieve= "PMachieve: Primary Mean Child Achievement"
           PMses=     "PMses: Primary Mean Child SES"
           PrimN=     "PrimN: N Obs per Primary School";
     DROP _TYPE_ _FREQ_; RUN;

TITLE "SAS Descriptives and Correlations for Primary Schools";
PROC MEANS NDEC=2  DATA=work.Primary; VAR PMachieve Pdenom PMses PrimN; RUN; 
PROC CORR NOSIMPLE DATA=work.Primary; VAR PMachieve Pdenom PMses PrimN; RUN;
TITLE;

* Get means per secondary school for child variables;
PROC SORT DATA=work.Example5; BY SschoolID; RUN;
PROC MEANS NOPRINT DATA=work.Example5; BY SschoolID Sdenom; VAR achieve ses; 
     OUTPUT OUT=work.Secondary MEAN(achieve ses)=SMachieve SMses; RUN;
DATA work.Secondary; SET work.Secondary;
     SecN = _FREQ_;
     LABEL SMachieve= "SMachieve: Secondary Mean Child Achievement"
           SMses=     "SMses: Secondary Mean Child SES"
           SecN=      "SecN: N Obs per Secondary School";
     DROP _TYPE_ _FREQ_; RUN;

TITLE "SAS Descriptives and Correlations for Secondary Schools";
PROC MEANS NDEC=2  DATA=work.Secondary; VAR SMachieve Sdenom SMses SecN; RUN;
PROC CORR NOSIMPLE DATA=work.Secondary; VAR SMachieve Sdenom SMses SecN; RUN;
TITLE;

* Get means per unique combination of primary/secondary school for child variables;
PROC SORT DATA=work.Example5; BY PschoolID SschoolID; RUN;
PROC MEANS NOPRINT DATA=work.Example5; BY PschoolID SschoolID Pdenom Sdenom; VAR achieve ses; 
     OUTPUT OUT=work.UniquePS MEAN(achieve ses)=PSMachieve PSMses; RUN;
DATA work.UniquePS; SET work.UniquePS;
     UniqueN = _FREQ_; UniqueID = _N_; * Create unique ID;
     LABEL PSMachieve= "PSMachieve: Unique Primary/Secondary Mean Child Achievement"
           PSMses=     "PSMses: Unique Primary/Secondary Mean Child SES"
           UniqueN=    "UniqueN: N Obs per Unique Primary/Secondary School"
		 UniqueID=   "UniqueID: Unique Primary/Secondary School ID";
     DROP _TYPE_ _FREQ_; RUN;

TITLE "SAS Descriptives and Correlations for Unique Primary/Secondary Combination";
PROC FREQ DATA=work.UniquePS; TABLE UniqueN; RUN;
PROC MEANS NDEC=2  DATA=work.UniquePS; VAR PSMachieve PSMses UniqueN; RUN; 
PROC CORR NOSIMPLE DATA=work.UniquePS; VAR PSMachieve PSMses UniqueN; RUN; 
TITLE;

* Merge all three cluster mean datasets back into individual data;
PROC SORT DATA=work.Example5; BY PschoolID; RUN;
DATA work.Example5; MERGE work.Example5 work.Primary; BY PschoolID; RUN;

PROC SORT DATA=work.Example5; BY SschoolID; RUN;
DATA work.Example5; MERGE work.Example5 work.Secondary; BY SschoolID; RUN;

PROC SORT DATA=work.Example5; BY PschoolID SschoolID; RUN;
DATA work.Example5; MERGE work.Example5 work.UniquePS; BY PschoolID SschoolID; RUN;

* Center and label predictors;
DATA work.Example5; SET work.Example5;
* Constant-centered predictors;
  ses4    = ses - 4;    
  PMses4  = PMses - 4;  
  SMses4  = SMses - 4; 
  PSMses4 = PSMses - 4; 
* Cluster-mean-centered level-1 child predictors;
  WPses  = ses - PMses;   * Within primary only;
  WSses  = ses - SMses;   * Within secondary only;
  WPSses = ses - PSMses;  * Within unique combination;
  LABEL WPses=  "WPses: Within-Primary Centered Child SES"
        WSses=  "WSses: Within-Secondary Centered Child SES"
        WPSses= "WPSses: Within-Primary/Secondary Centered Child SES"; RUN;

TITLE "SAS Descriptives and Correlations in Child-Level Data";
PROC MEANS VAR DATA=work.Example5; VAR WPSses; RUN;
PROC CORR NOSIMPLE DATA=work.Example5; 
     VAR PMses SMses PSMses WPses WSses WPSses; RUN; TITLE;


***********************************************************************************;
*******                 BEGIN EMPTY MODELS FOR EXAMPLE 5                    *******;
***********************************************************************************;

TITLE "SAS Model 1a: Empty Means, Primary Random Intercept Only";
PROC MIXED DATA=work.Example5 COVTEST NOCLPRINT IC NAMELEN=100 METHOD=REML;
     CLASS PschoolID SschoolID;
     MODEL achieve =  / SOLUTION DDFM=Satterthwaite; 
     RANDOM INTERCEPT / SUBJECT=PschoolID TYPE=UN; * L2 primary variance; 
     ODS OUTPUT InfoCrit=FitEmpty1; * Save output for LRT;
 RUN; TITLE;

TITLE "SAS Model 1b: Empty Means, Primary by Secondary Crossed Random Intercepts";
PROC MIXED DATA=work.Example5 COVTEST NOCLPRINT IC NAMELEN=100 METHOD=REML;
     CLASS PschoolID SschoolID;
     MODEL achieve =  / SOLUTION DDFM=Satterthwaite OUTPM=PredEmpty; 
     RANDOM INTERCEPT / SUBJECT=PschoolID TYPE=UN; * L2 primary variance; 
     RANDOM INTERCEPT / SUBJECT=SschoolID TYPE=UN; * L2 secondary variance; 
     ODS OUTPUT InfoCrit=FitEmpty2 CovParms=CovEmpty2; * Save output for LRT and pseudo-R2;
 RUN; TITLE;
* Calculate difference in model fit relative to model 1a;
%FitTest(FitFewer=FitEmpty1, FitMore=FitEmpty2);

* Compute proportion of variance of each kind;
DATA work.PropVar; SET work.CovEmpty2; IF CMISS(Subject)=0 THEN CovParm=Subject; RUN; 
PROC TRANSPOSE DATA=work.PropVar OUT=work.Prop; ID CovParm; VAR Estimate; RUN; 
DATA work.Prop; SET work.Prop; Total = SUM(PschoolID, SschoolID, Residual); 
     PropL2Primary = PschoolID/Total; PropL2Second = SschoolID/Total; 
	PropResid=Residual/Total; DROP _NAME_; RUN;
TITLE "Empty Model Proportions of Variance";
PROC PRINT NOOBS DATA=work.Prop; RUN; TITLE;

TITLE "SAS Model 1c: Empty Means, Primary by Secondary and Unique Crossed Random Intercepts";
PROC MIXED DATA=work.Example5 COVTEST NOCLPRINT IC NAMELEN=100 METHOD=REML;
     CLASS PschoolID SschoolID;
     MODEL achieve =  / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / SUBJECT=SschoolID TYPE=UN; * L2 secondary variance; 
     RANDOM INTERCEPT / SUBJECT=PschoolID TYPE=UN; * L2 primary variance; 
     RANDOM INTERCEPT / SUBJECT=PschoolID*SschoolID TYPE=UN; * L2 unique combo variance;
     ODS OUTPUT InfoCrit=FitEmpty3; * Save output for LRT; 
 RUN; TITLE;
* Calculate difference in model fit relative to model 1b;
%FitTest(FitFewer=FitEmpty2, FitMore=FitEmpty3);

TITLE "SAS Empty Means, Three-Way Crossed Model for SES Predictor";
PROC MIXED DATA=work.Example5 COVTEST NOCLPRINT IC NAMELEN=100 METHOD=REML;
     CLASS PschoolID SschoolID;
     MODEL ses =      / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / SUBJECT=SschoolID TYPE=UN; * L2 secondary variance; 
     RANDOM INTERCEPT / SUBJECT=PschoolID TYPE=UN; * L2 primary variance; 
     RANDOM INTERCEPT / SUBJECT=PschoolID*SschoolID TYPE=UN; * L2 unique combo variance;
 RUN; TITLE;


***********************************************************************************;
*******               BEGIN CONDITIONAL MODELS FOR EXAMPLE 5                *******;
***********************************************************************************;

TITLE "SAS Model 2: Model 1b + Primary*Secondary School Denomination";
PROC MIXED DATA=work.Example5 COVTEST NOCLPRINT IC NAMELEN=100 METHOD=REML;
     CLASS PschoolID SschoolID;
     MODEL achieve = Pdenom Sdenom Pdenom*Sdenom / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / SUBJECT=PschoolID TYPE=UN; * L2 primary variance; 
     RANDOM INTERCEPT / SUBJECT=SschoolID TYPE=UN; * L2 secondary variance;
     ODS OUTPUT CovParms=CovPxSdenom; * Save output for pseudo-R2;
 RUN; TITLE;
* Pseudo-R2 relative to empty means, crossed random intercepts model 1b;
%PseudoR2(NCov=3, CovFewer=CovEmpty2, CovMore1=CovPxSdenom);

TITLE "SAS Model 3a: Add Child SES";
PROC MIXED DATA=work.Example5 COVTEST NOCLPRINT IC NAMELEN=100 METHOD=REML;
     CLASS PschoolID SschoolID;
     MODEL achieve = Pdenom Sdenom Pdenom*Sdenom 
                     ses4 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / SUBJECT=PschoolID TYPE=UN; * L2 primary variance; 
     RANDOM INTERCEPT / SUBJECT=SschoolID TYPE=UN; * L2 secondary variance; 
     ODS OUTPUT CovParms=CovSES1; * Save output for pseudo-R2;
 RUN; TITLE;
* Calculate change in PseudoR2 relative to denom model 2;
%PseudoR2(NCov=3, CovFewer=CovEmpty2, CovMore1=CovPxSdenom, CovMore2=CovSES1);

TITLE "SAS Model 3b: Add Child SES Contextual Effects";
PROC MIXED DATA=work.Example5 COVTEST NOCLPRINT IC NAMELEN=100 METHOD=REML;
     CLASS PschoolID SschoolID;
     MODEL achieve = Pdenom Sdenom Pdenom*Sdenom 
                     ses4 PMses4 SMses4 PSMses4 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / SUBJECT=PschoolID TYPE=UN; * L2 primary variance; 
     RANDOM INTERCEPT / SUBJECT=SschoolID TYPE=UN; * L2 secondary variance;
     ODS OUTPUT InfoCrit=FitFix CovParms=CovSES2; * Save output for LRT and pseudo-R2;
 RUN; TITLE;
* Calculate change in PseudoR2 relative to denom model 2;
%PseudoR2(NCov=3, CovFewer=CovEmpty2, CovMore1=CovPxSdenom, CovMore2=CovSES2);

TITLE "SAS Model 4a: Add Random WPS-Centered Child SES across Primary Schools";
PROC MIXED DATA=work.Example5 COVTEST NOCLPRINT IC NAMELEN=100 METHOD=REML;
     CLASS PschoolID SschoolID;
     MODEL achieve = Pdenom Sdenom Pdenom*Sdenom 
                     ses4 PMses4 SMses4 PSMses4 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT WPSses / SUBJECT=PschoolID TYPE=UN; * L2 primary variances; 
     RANDOM INTERCEPT        / SUBJECT=SschoolID TYPE=UN; * L2 secondary variance; 
     ODS OUTPUT InfoCrit=FitRandP; * Save output for LRT;
 RUN; TITLE;
* Calculate difference in model fit relative to fixed SES model 3b;
%FitTest(FitFewer=FitFix, FitMore=FitRandP);

TITLE "SAS Model 4b: Model 3b + Random WPS-Centered Child SES across Secondary Schools";
PROC MIXED DATA=work.Example5 COVTEST NOCLPRINT IC NAMELEN=100 METHOD=REML;
     CLASS PschoolID SschoolID;
     MODEL achieve = Pdenom Sdenom Pdenom*Sdenom 
                     ses4 PMses4 SMses4 PSMses4 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT        / SUBJECT=PschoolID TYPE=UN; * L2 primary variance; 
     RANDOM INTERCEPT WPSses / SUBJECT=SschoolID TYPE=UN; * L2 secondary variances; 
     ODS OUTPUT InfoCrit=FitRandS; * Save output for LRT;
 RUN; TITLE;
* Calculate difference in model fit relative to random SES model 3c;
%FitTest(FitFewer=FitFix, FitMore=FitRandS);

* Close output;
ODS RTF CLOSE;



